require 'rubygems'
require 'yaml'
require 'erb'
require 'rexml/document'
require 'sinatra'

get '/' do
   log = load_log
   apply_template log
end

post '/' do
   save_log params
   redirect '/'
end

def load_log
   log = ""

   f = File.open "bbslog.yml"
   f.flock File::LOCK_SH
   log = YAML.load f.read
   f.close
   
   return log
rescue
   return ""
end

def apply_template(log)
   template = nil
   
   f = File.open "public/template.html"
   f.flock File::LOCK_SH
   template = REXML::Document.new f
   f.close

   article_template = get_by_class(template, 'article')
   log.each do |entry|
      new_article = article_template.deep_clone
      entry.keys.each do |key|
         if element = get_by_class(new_article, key)
            if key == "_content"
               insert_content(element, entry[key])
            else
               element.text = entry[key]
            end
         end
      end
      article_template.parent.insert_after(article_template, new_article)
   end
   article_template.remove

   colors = %w[#000000 #666666 #663333 #CC3333 #CC6600 #336633 #33CC33 #CC33FF #990099 #3333CC #66AAFF #FF66C0 #FF8800];
   color_template = get_by_class(template, 'input_color')
   colors.each do |cl|
      new_color = color_template.deep_clone
      new_color.elements['span'].add_attribute('style', "color:#{cl}")
      new_color.add_attribute('value', cl)
      color_template.parent.add_element(new_color, color_template)
   end
   color_template.remove

   #elem = REXML::Element.new("<b>aaaaa</b>")
   #template.add_emement(elem, article_template)
   
   template.to_s
end

def save_log(params)
   f = File.open "bbslog.erb"
   f.flock File::LOCK_SH
   erb = ERB.new f.read
   f.close

   num = get_next_number
   indented_content = params[:_content].gsub(/^/, "  ")
   timestamp = Time.now.strftime("%Y/%m/%d %H:%M:%S")
   log = erb.result(binding)

   f = File.open "bbslog.yml", "a"
   f.flock File::LOCK_EX
   f.write(log)
   f.close
end

def get_by_class(element, class_name)
   element.elements["//*[@class='#{class_name}']"]
end

def get_next_number
   log = load_log
   begin
      return log.sort_by{|v| v["_num"]}.last["_num"] + 1
   rescue
      return 0
   end
end

def insert_content(element, content)
   lines = content.split("\n")
   lines.each do |line|
      element.add_text line
      element.add_element "br"
   end
end
